# 1時間毎のアクセス数を送信元IPアドレス別にグラフ表示するスクリプト
# access_logは1日分（00:00～23:59）が1ファイルである
# access_logのフォーマットはCombined Log Format
library(ggplot2)
library(ggiraph)
library(dplyr)
library(data.table)

# access_logファイル名の設定
file_name <- c("./sample_log/access_log.20160606")

# testdataという変数をdata.table型で宣言
testdata <- data.table()

# access_logファイルの読み込み
# sep=で列の区切り文字を指定。今回はスペース
# colClasses=で各列のデータ型を指定
# col.names=で各列の列名を指定
testdata <- fread(file_name,
                  sep=" ",
                  colClasses=c(V1="character", V2="character", V3="character", 
                               V4="character", V5="character",
                               V6="character", V7="integer", V8="character",
                               V9="character", V10="character"),
                  col.names=c(V1="source_ip", V2="client_id", V3="auth_id",
                              V4="date", V5="date_tmp",
                              V6="request", V7="status_code", V8="obj_size",
                              V9="referer", V10="user_agent"))

# date列に不要な文字列が残るため削除
# gsub("置換対象文字", "置換後文字", 対象となるデータ)
testdata$date <- gsub("\\[", "", testdata$date)

# TIMEロケールをCに変更
# 次の日時データの型変換でエラーにならないようにするため
Sys.setlocale("LC_TIME", "C")

# date列に格納された日時を変換
# date列から日付のみ、時のみ、時分のみ、時分秒のみを取り出して
# testdataに列を追加する
# mutate(列名 = 値)：列を追加
# %>% はLinuxコマンドの¦(パイプ)のような役割をする。dplyrパッケージで提供
testdata <- testdata %>%
  mutate(date    = as.POSIXct(date, "JST", format="%d/%b/%Y:%H:%M:%S")) %>%
  mutate(daily   = as.Date(date, "%y%m%d")) %>%
  mutate(hourly  = as.POSIXct(trunc.POSIXt(date, "hours"))) %>%
  mutate(minute  = as.POSIXct(trunc.POSIXt(date, "mins"))) %>%
  mutate(second  = as.POSIXct(trunc.POSIXt(date, "secs")))

# testdataから分析をしてグラフを作成する
# select(hourly, source_ip)：処理量を減らすためhourly、source_ip列のみを抽出
# group_by(hourly, source_ip)：hourly且つsource_ipでグルーピングする
# この場合、1時間毎にsource_ip別にグルーピング
# summarise(count=n())：グルーピングした中で数をカウント
# この場合、1時間毎にsource_ip別に何回出現したかカウント
# ggplot(aes(x=hourly, y=count, fill=source_ip, tooltip=source_ip))
# ：グラフをプロットする。x=x軸のデータを指定、y=y軸のデータを指定
# fill=source_ip別に塗りつぶしの色を指定
# tooltip=インタラクティブ表示するデータを指定
# theme(legend.position="none")：凡例を表示しない
# geom_bar_interactive(stat = "identity")：インタラクティブな棒グラフの描画を指定
# ggiraph(code=print(testdata_graph))：testdata_graphをグラフ描画する

testdata %>%
  select(hourly, source_ip) %>%
  #filter(source_ip!="監視用のIPアドレス") %>%
  group_by(hourly, source_ip) %>%
  summarise(count=n()) %>%
  ggplot(aes(x=hourly, y=count, fill=source_ip, tooltip=source_ip))+
  theme(legend.position="none")+
  geom_bar_interactive(stat = "identity") -> testdata_graph
ggiraph(code=print(testdata_graph))
